package DICDB;
use base 'Class::DBI';
sub connect {
  shift;
  __PACKAGE__->connection(@_, { ChopBlanks=>0, AutoCommit=>1 });
}

package DICDB::Dictionary;
use base 'DICDB';
__PACKAGE__->columns(Primary => qw/word_index/);
__PACKAGE__->columns(All => qw/word_index word meaning date/);
__PACKAGE__->set_sql(auto_complete => <<"");
SELECT word_index, meaning FROM __TABLE__ WHERE word LIKE ? ORDER BY word LIMIT 10

package DIC;
use strict;
use warnings;
use Encode;
use HTTP::Date;
use LWP::UserAgent;
use CGI qw/:cgi/;

sub new {
  my $class = shift;
  my $self = bless {}, $class;

  $self->init;
  my $settings = shift;
  $self->setting($settings) if ref $settings eq 'HASH';

  return $self;
}

sub init {
  my $self = shift;

  $self->{cgi} ||= CGI->new;
  $self->{cgi}->charset('utf-8');

  $self->{ua} ||= LWP::UserAgent->new;
  $self->{ua}->timeout(10);
  $self->{ua}->agent('Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.7.5) Gecko/20041108 Firefox/1.0');

  $self->{message} = '';
}

sub init_db {
  my $self = shift;
  DICDB::Dictionary->table($self->{settings}->{db_table} || "dictionary");
  DICDB->connect("$self->{settings}->{db_driver}:$self->{settings}->{db_name}",
                 $self->{settings}->{db_user}, $self->{settings}->{db_pass});

  if($self->{settings}->{mysql_4_1}) {
    DICDB->db_Main()->do("SET NAMES utf8");
    DICDB->db_Main()->do("SET CHARACTER SET utf8");
  }
}

sub setting {
  my $self = shift;
  my $settings = shift;

  map { $self->{settings}->{$_} = $settings->{$_} } keys %$settings;

  1;
}

sub auto_complete {
  my $self = shift;
  my $q = $self->{cgi};

  $self->init_db;
  
  my $word = $q->param($self->{settings}->{textbox_name} || 'word');
  my @result;
  if($word) {
    @result = DICDB::Dictionary->search_auto_complete("$word%");
    foreach my $colum (@result) {
      my $word = $colum->word_index;
      my $meaning = $colum->meaning;
      $meaning =~ s!\n!<br />!gs;
      $meaning =~ s!(【.*?】)!<span class="class">$1</span>!gms;
      $self->{message} .= qq!<li><span class="word">$word</span><br />$meaning</li>!;
    }
  }
  
  print $q->header;
  print qq!<ul>$self->{message}</li>! if $self->{message};

  1;
}

sub fetch {
  my $self = shift;
  my $ua = $self->{ua};
  my $q = $self->{cgi};

  $self->init_db;
  
  my $word = $q->param($self->{settings}->{textbox_name} || 'word');
  my @result = DICDB::Dictionary->search_auto_complete("$word");
  
  if($word and !@result) {
    my $response = $ua->get("http://www2.alc.co.jp/ejr/index.php?word_in=$word&word_in2=reedeirrf&word_in3=zJPa7DCxJ15687987t");
    if($response->is_success) {
      $self->{message} = 0;
      $self->add_dictionary($response->content);
    }
  }

  print $q->header;
  print $self->{message};
}

sub add_dictionary {
  my $self = shift;
  my $html = shift;

  Encode::from_to($html, "shiftjis", "utf8");
  while($html =~ m!<li><span class="ejr_e">(.*?)</span><br>(.*?)</li>!mgs) {
    my ($word, $meaning) = ($1, $2);

    $word =~ s/[\r\n\t]//mgs;
    $word =~ s/<.*?>//mgs;
    $meaning =~ s/[\r\n\t]//mgs;
    $meaning =~ s/<(→.*?)>/&lt;$1&gt;/g;
    $meaning =~ s/<br.*?>/\n/g;
    $meaning =~ s/<.*?>//mgs;

    $self->{message}++;

    eval {
      DICDB::Dictionary->create({
        word_index => $word,
        word => $word,
        meaning => qq!$meaning!,
        date => HTTP::Date::time2iso(time),
      });
    };
  }

  1;
}

if(!$INC{'DIC.pm'}) {
  package main;

  my $dic = DIC->new;

  my $settings = {
    db_driver => 'dbi:mysql',
    db_name => '',
    db_user => '',
    db_pass => '',
    db_table => 'my_dictionary',
  };

  $dic->setting($settings);

  $dic->auto_complete; # $dic->fetch

}

1;